#include <stdlib.h>
#include <qsettings.h>
#include <qdatetime.h>
#include <qdir.h>

#include "gconfig.h"

/*!	\class GConfig
	\brief The Gloster configuration class. It centralized all accesses to the Gloster configuration.

	This object is the first created when the application is launched. It holds very important information.
	All the configuration is stored in an INI file in a specific directory depending on the platform :

	- On a Windows system it is stored in "\%HOME\Application data\GoatSoft\Gloster".
	- On Linux system it is stored in "%HOME\.qt\glosterrc"

	This object manage also the files types and the other users's public keys/nodeids.
*/

//! A GConfig constructor
/*! Create a GConfig and load everything the GConfig can hold :
- the Gloster configuration
- other users's public keys/nodeids
- file types

You should always use this constructor.

*/ 
GConfig::GConfig():QObject(0),Translator(0)
{
	QString confdir;

	applicationdirpath = QDir::convertSeparators(qApp->applicationDirPath()) + QDir::separator();
	#ifdef Q_WS_WIN
		confdir = QString(getenv("APPDATA"))+ QDir::separator() + G_DOMAIN + QDir::separator() + G_PRODUCT + QDir::separator();
	#else
		confdir = QDir::homeDirPath() + "/.Gloster/";
	#endif
	qWarning(confdir);
	if(validateDir(confdir))
		ConfDir = confdir;
	else
	{
		qWarning(tr("Attention : le rpertoire de configuration ne peut etre cr. Il a t dplac automatiquement : ")+applicationdirpath);
		ConfDir = applicationdirpath;
	}
	reload();
	initfcat();
}

//! The GConfig destructor
GConfig::~GConfig()
{

}

//! Returns TRUE if Gloster have been configured yet
//!\sa setIsConfigured(bool b)
bool GConfig::isConfigured()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readBoolEntry("/Gloster/Settings/general/isconfigured");
}

//! Returns the last x coordinate on the screen of the main window
int GConfig::xPos()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readNumEntry("/Gloster/Settings/window/xpos");
}

//! Returns the last y coordinate on the screen of the main window
int GConfig::yPos()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readNumEntry("/Gloster/Settings/window/ypos");
}

//! Returns the last width of the main window
int GConfig::wSize()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readNumEntry("/Gloster/Settings/window/wsize");
}

//! Returns the last height of the main window
int GConfig::hSize()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readNumEntry("/Gloster/Settings/window/hsize");
}

//! Returns TRUE if the main window was maximized
bool GConfig::isMaximized()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readBoolEntry("/Gloster/Settings/window/ismaximized");
}

//! Returns the known nodes IP
QStringList GConfig::ipList()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readListEntry("/Gloster/Settings/connection/iplist");
}

//! Returns the known nodes ports
QStringList GConfig::portList()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	return settings.readListEntry("/Gloster/Settings/connection/portlist");
}


//! Returns the user's public key
QString GConfig::pubKey(const QString& nodeid)
{
	QMap<QString,QString>::iterator result;
	result = Keys.find(nodeid);
	if(result == Keys.end())
		return QString::null;
	return *result;
}

//! Defines the configuration state of Gloster. If b is TRUE, Gloster will be considered as configured
//!\sa isConfigured()
void GConfig::setIsConfigured(bool b)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/isconfigured",b);
}

//! Defines the x coordinate on the screen of the main window
void GConfig::setXPos(int x)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/window/xpos",x);
}

//! Defines the y coordinate on the screen of the main window
void GConfig::setYPos(int y)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/window/ypos",y);
}

//! Defines the width of the main window
void GConfig::setWSize(int w)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/window/wsize",w);
}

//! Defines the height of the main window
void GConfig::setHSize(int h)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/window/hsize",h);
}


void GConfig::setIsMaximized(bool b)
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/window/ismaximized",b);
}

void GConfig::setNickname(const QString& nickname)
{
	QSettings settings(QSettings::Ini);

	Nick = nickname;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/Nick",nickname);
}

void GConfig::setLanguage(const QString& lang)
{
	QSettings settings(QSettings::Ini);

	Language = lang;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/language",lang);
}

void GConfig::setNetKey(const QString& key)
{
	QSettings settings(QSettings::Ini);

	NetKey = key;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/netkey",key);
}

void GConfig::setSplashScreen(bool b)
{
	QSettings settings(QSettings::Ini);

	SplashScreen = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/splash",b);
}

void GConfig::setTextUnderIcons(bool b)
{
	QSettings settings(QSettings::Ini);

	TextUnderIcons = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/general/textundericons",b);

	emit updateConfTextUnderIcons(b);
}


void GConfig::setPublicIp(const QString& ip)
{
	QSettings settings(QSettings::Ini);

	PublicIp = ip;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/publicip",ip);
}

void GConfig::setConSpeed(int s)
{
	QSettings settings(QSettings::Ini);

	ConSpeed = s;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/conspeed",s);
}

void GConfig::setListenPort(int port)
{
	QSettings settings(QSettings::Ini);

	ListenPort = port;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/listenport",port);
}

void GConfig::setSlotCount(int n)
{
	QSettings settings(QSettings::Ini);

	SlotCount = n;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/slotcount",n);
}

void GConfig::setConType(int n)
{
	QSettings settings(QSettings::Ini);

	ConType = n;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/contype",n);
}

void GConfig::setAutoCon(bool b)
{
	QSettings settings(QSettings::Ini);

	AutoCon = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/autocon",b);

	emit updateConfAutoCon();
}

void GConfig::setKeepCon(int n)
{
	QSettings settings(QSettings::Ini);

	KeepCon = n;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/keepcon",n);

	emit updateConfKeepCon();
}

void GConfig::setIpList(const QStringList& l)
{
	QSettings settings(QSettings::Ini);

	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/iplist",l);
}

void GConfig::setPortList(const QStringList& l)
{
	QSettings settings(QSettings::Ini);

	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/connection/portlist",l);
}

void GConfig::setAutoRefreshShares(bool b)
{
	QSettings settings(QSettings::Ini);

	if(AutoRefreshShares!=b)
	{
		AutoRefreshShares = b;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/folders/autorefreshshares",b);

		emit updateConfRefreshShareList();
	}
}

void GConfig::setAutoRefreshUnit(int unit)
{
	QSettings settings(QSettings::Ini);

	if(AutoRefreshUnit != unit)
	{
		AutoRefreshUnit = unit;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/folders/autorefreshunit",unit);

		emit updateConfRefreshShareList();
	}
}

void GConfig::setAutoRefreshDelay(int n)
{
	QSettings settings(QSettings::Ini);

	if(AutoRefreshDelay!=n)
	{
		AutoRefreshDelay = n;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/folders/autorefreshdelay",n);

		emit updateConfRefreshShareList();
	}
}

bool GConfig::setDownloadDir(const QString& d)
{
	QSettings settings(QSettings::Ini);

	DownloadDir = QDir::cleanDirPath(d + QDir::separator());
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/folders/download",DownloadDir);

	return validateDir(d);
}

bool GConfig::setTempDir(const QString& d)
{
	QSettings settings(QSettings::Ini);

	TempDir = QDir::cleanDirPath(d + QDir::separator());
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/folders/temp",TempDir);

	return validateDir(d);
}

void GConfig::setAutoAcceptKey(bool b)
{
	QSettings settings(QSettings::Ini);

	AutoAcceptKey = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/encryption/autoacceptkey",b);
}

void GConfig::setMyPrivKey(const QString& key)
{
	QSettings settings(QSettings::Ini);

	MyPrivKey = key;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/encryption/privkey",key);
}


void GConfig::setMyPubKey(const QString& key)
{
	QSettings settings(QSettings::Ini);

	MyPubKey = key;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/encryption/pubkey",key);
}

void GConfig::addPubKey(const QString& nodeid, const QString& key, bool save)
{
	Keys.insert(nodeid,key);
	if(save)
		saveKeys();
}

void GConfig::setWebActivated(bool b)
{
	QSettings settings(QSettings::Ini);

	if(WebActivated != b)
	{
		WebActivated = b;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/webserver/activated",b);

		emit updateConfWebserver();
	}
}

void GConfig::setWebPassword(const QString& pass)
{
	QSettings settings(QSettings::Ini);

	if(WebPassword != pass)
	{
		WebPassword = pass;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/webserver/password",pass);

		emit updateConfWebserver();
	}
}

void GConfig::setWebListenPort(int n)
{
	QSettings settings(QSettings::Ini);

	if(WebListenPort!=n)
	{
		WebListenPort = n;
		settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
		settings.writeEntry("/Gloster/Settings/webserver/listenport",n);

		emit updateConfWebserver();
	}
}

void GConfig::setChatTextColor(const QColor& color)
{
	QSettings settings(QSettings::Ini);

	ChatTextColor = color;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/textcolor",colorToString(color));
}


void GConfig::setIsChatTextBold(bool b)
{
	QSettings settings(QSettings::Ini);

	ChatTextBold = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/textbold",b);
}

void GConfig::setIsChatTextItalic(bool b)
{
	QSettings settings(QSettings::Ini);

	ChatTextItalic = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/textitalic",b);
}

void GConfig::setIsChatTextUnderlined(bool b)
{
	QSettings settings(QSettings::Ini);

	ChatTextUnderlined = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/textunderlined",b);
}

void GConfig::setIsChatTimeStamp(bool b)
{
	QSettings settings(QSettings::Ini);

	ChatTimeStamp = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/chattimestamp",b);
}

void GConfig::setIsChanTimeStamp(bool b)
{
	QSettings settings(QSettings::Ini);

	ChanTimeStamp = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/chantimestamp",b);
}

void GConfig::setIsCustomTimeStamp(bool b)
{
	QSettings settings(QSettings::Ini);

	CustomTimeStamp = b;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/customtimestamp",b);
}

void GConfig::setTimeStampString(const QString &ts)
{
	QSettings settings(QSettings::Ini);

	TimeStampString = ts;
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.writeEntry("/Gloster/Settings/chat/timestampstring",ts);
}

unsigned int GConfig::defineFileType(const QString& ext)
{
	u_fcatResult r;

	r.p = cat.find(ext);
	return r.c;
}

void GConfig::writeTypes()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.beginGroup("/Gloster");
	settings.beginGroup("/ftypes");
	settings.writeEntry("/ace",1);
	settings.writeEntry("/avi",7);
	settings.writeEntry("/bmp",4);
	settings.writeEntry("/bz2",1);
	settings.writeEntry("/cue",3);
	settings.writeEntry("/doc",6);
	settings.writeEntry("/exe",5);
	settings.writeEntry("/gif",4);
	settings.writeEntry("/gz",1);
	settings.writeEntry("/htm",6);
	settings.writeEntry("/html",6);
	settings.writeEntry("/iso",3);
	settings.writeEntry("/jpg",4);
	settings.writeEntry("/mdf",3);
	settings.writeEntry("/mkv",7);
	settings.writeEntry("/mp3",2);
	settings.writeEntry("/mpc",2);
	settings.writeEntry("/mpeg",7);
	settings.writeEntry("/mpg",7);
	settings.writeEntry("/nrg",3);
	settings.writeEntry("/ogg",2);
	settings.writeEntry("/png",4);
	settings.writeEntry("/psd",4);
	settings.writeEntry("/rar",1);
	settings.writeEntry("/sh",5);
	settings.writeEntry("/tar",1);
	settings.writeEntry("/tga",4);
	settings.writeEntry("/tiff",4);
	settings.writeEntry("/txt",6);
	settings.writeEntry("/wav",2);
	settings.writeEntry("/wma",2);
	settings.writeEntry("/wmv",7);
	settings.writeEntry("/mov",7);
	settings.writeEntry("/zip",1);
}

void GConfig::saveKeys()
{
	QStringList PubKeys,NodeIds;
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
        QMap<QString,QString>::Iterator it;
        for ( it = Keys.begin(); it != Keys.end(); ++it )
	{
                    NodeIds << it.key();
                    PubKeys << it.data();
	}
	settings.writeEntry("/Gloster/Settings/encryption/pubkeys",PubKeys);
	settings.writeEntry("/Gloster/Settings/encryption/nodeids",NodeIds);
}

void GConfig::save()
{
	saveConf();
	saveKeys();
}

void GConfig::saveConf()
{
	QStringList PubKeys,NodeIds;
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.beginGroup("/Gloster");
	settings.beginGroup("/Settings");

	settings.writeEntry("/general/Nick",Nick);
	settings.writeEntry("/general/language",Language);
	settings.writeEntry("/general/netkey",NetKey);
	settings.writeEntry("/general/splash",SplashScreen);
	settings.writeEntry("/general/textundericons",TextUnderIcons);
	settings.writeEntry("/general/isconfigured",TRUE);

	settings.writeEntry("/connection/publicip",PublicIp);
	settings.writeEntry("/connection/conspeed",ConSpeed);
	settings.writeEntry("/connection/listenport",ListenPort);
	settings.writeEntry("/connection/slotcount",SlotCount);
	settings.writeEntry("/connection/contype",ConType);
	settings.writeEntry("/connection/autocon",AutoCon);
	settings.writeEntry("/connection/keepcon",KeepCon);

	settings.writeEntry("/folders/autorefreshshares",AutoRefreshShares);
	settings.writeEntry("/folders/autorefreshunit",AutoRefreshUnit);
	settings.writeEntry("/folders/autorefreshdelay",AutoRefreshDelay);
	settings.writeEntry("/folders/download",DownloadDir);

	settings.writeEntry("/encryption/autoacceptkey",AutoAcceptKey);
        QMap<QString,QString>::Iterator it;
        for ( it = Keys.begin(); it != Keys.end(); ++it )
	{
                    NodeIds << it.key();
                    PubKeys << it.data();
	}
	settings.writeEntry("/encryption/pubkeys",PubKeys);
	settings.writeEntry("/encryption/nodeids",NodeIds);
	settings.writeEntry("/encryption/pubkey",MyPubKey);
	settings.writeEntry("/encryption/privkey",MyPrivKey);

	settings.writeEntry("/webserver/activated",WebActivated);
	settings.writeEntry("/webserver/password",WebPassword);
	settings.writeEntry("/webserver/listenport",WebListenPort);

	settings.writeEntry("/chat/textcolor",colorToString(ChatTextColor));
	settings.writeEntry("/chat/textbold",ChatTextBold);
	settings.writeEntry("/chat/textitalic",ChatTextItalic);
	settings.writeEntry("/chat/textunderlined",ChatTextUnderlined);
	settings.writeEntry("/chat/chattimestamp",ChatTimeStamp);
	settings.writeEntry("/chat/chantimestamp",ChanTimeStamp);
	settings.writeEntry("/chat/customtimestamp",CustomTimeStamp);
	settings.writeEntry("/chat/timestampstring",TimeStampString);

}

void GConfig::reload()
{
	loadConf();
	loadPublicKeys();
}

void GConfig::loadConf()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	settings.beginGroup("/Gloster");
	settings.beginGroup("/Settings");

	validateDir(confDir());

	/* Options generales */
	Nick = settings.readEntry("/general/Nick");
	Language = settings.readEntry("/general/language");
	NetKey = settings.readEntry("/general/netkey");
	SplashScreen = settings.readBoolEntry("/general/splash");
	TextUnderIcons = settings.readBoolEntry("/general/textundericons");

	/* Options de connexion */
	PublicIp = settings.readEntry("/connection/publicip");
	ConSpeed = settings.readNumEntry("/connection/conspeed");
	ListenPort = settings.readNumEntry("/connection/listenport");
	SlotCount = settings.readNumEntry("/connection/slotcount");
	ConType = settings.readNumEntry("/connection/contype");
	AutoCon = settings.readBoolEntry("/connection/autocon");
	KeepCon = settings.readNumEntry("/connection/keepcon");

	/* Options des repertoires */
	AutoRefreshShares = settings.readBoolEntry("/folders/autorefreshshares");
	AutoRefreshUnit = settings.readNumEntry("/folders/autorefreshunit");
	AutoRefreshDelay = settings.readNumEntry("/folders/autorefreshdelay");
	DownloadDir = settings.readEntry("/folders/download");
	TempDir = settings.readEntry("/folders/temp");
	if(!validateDir(DownloadDir) && isConfigured())
	{
		qWarning(tr("Attention : le rpertoire de destination ne peut tre cr. Il a t deplac automatiquement")+" : "+confDir()+"incoming");
		setDownloadDir(confDir()+"incoming");
	}
	if(!validateDir(TempDir) && isConfigured())
	{
		qWarning(tr("Attention : le rpertoire temporaire ne peut tre cr. Il a t deplac automatiquement")+" : "+confDir()+"temp");
		setTempDir(confDir()+"temp");
	}

	/* Options de cryptage */
	AutoAcceptKey = settings.readBoolEntry("/encryption/autoacceptkey");
	MyPrivKey = settings.readEntry("/encryption/privkey");
	MyPubKey = settings.readEntry("/encryption/pubkey");

	/* Options du serveur web */
	WebActivated = settings.readBoolEntry("/webserver/activated");
	WebPassword = settings.readEntry("/webserver/password");
	WebListenPort = settings.readNumEntry("/webserver/listenport");

	/* Options des fenetres de chat */
	stringToColor(settings.readEntry("/chat/textcolor"),ChatTextColor);
	ChatTextBold = settings.readBoolEntry("/chat/textbold");
	ChatTextItalic = settings.readBoolEntry("/chat/textitalic");
	ChatTextUnderlined = settings.readBoolEntry("/chat/textunderlined");
	ChatTimeStamp = settings.readBoolEntry("/chat/chattimestamp");
	ChanTimeStamp = settings.readBoolEntry("/chat/chantimestamp");
	CustomTimeStamp = settings.readBoolEntry("/chat/customtimestamp");
	TimeStampString = settings.readEntry("/chat/timestampstring");

}

void GConfig::initfcat()
{
	QSettings settings(QSettings::Ini);
	QStringList l;
	u_fcatResult i;

	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	l = settings.entryList("/Gloster/ftypes");
	for ( QStringList::Iterator it = l.begin(); it != l.end(); ++it )
	{
		i.c = settings.readNumEntry("/Gloster/ftypes/"+*it,0);
	      	cat.insert((*it), i.p);
	}
}

void GConfig::loadPublicKeys()
{
	QSettings settings(QSettings::Ini);
	settings.setPath(G_DOMAIN,G_PRODUCT,QSettings::User);
	QStringList PubKeys = settings.readListEntry("/Gloster/Settings/encryption/pubkeys");
	QStringList NodeIds = settings.readListEntry("/Gloster/Settings/encryption/nodeids");

	Keys.clear();
	QStringList::Iterator it2 = PubKeys.begin();
	for( QStringList::Iterator it = NodeIds.begin(); it != NodeIds.end(); ++it,++it2 )
	{
		if(!(*it).isEmpty() && !(*it2).isEmpty())
			Keys.insert(*it,*it2);
	}
}

QString GConfig::colorToString(const QColor& color)
{
	return (QString::number(color.red())+";"+QString::number(color.green())+";"+QString::number(color.blue()));
}

void GConfig::stringToColor(const QString& str, QColor& color)
{
	color.setRgb(str.section(';',0,0).toInt(),str.section(';',1,1).toInt(),str.section(';',2,2).toInt());
}

bool GConfig::validateDir(const QString& dir)
{
	QFileInfo fd(dir);
	QDir d(dir);
	QStringList temp = QStringList::split(QDir::separator(),d.absPath(),TRUE);
	QString path = "";
	int i=-1;

	QStringList::Iterator it;
	for ( it = temp.begin(); it != temp.end(); ++it )
	{
		d.setPath(path+(*it));
		if(!d.exists())
			break;
		path += *it+QDir::separator();
		++i;
	}
	while(it != temp.end())
	{
		path += *it+QDir::separator();
		if(!d.mkdir(path,TRUE))
			return FALSE;
		++it;
	}
	if(!fd.isWritable())
		return FALSE;
	return TRUE;
}
