#include "dirview.h"
#include "vdirview.h"

#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qpoint.h>
#include <qmessagebox.h>
#include <qdragobject.h>
#include <qmime.h>
#include <qstrlist.h>
#include <qstringlist.h>
#include <qapplication.h>
#include <qheader.h>
#include <qdeepcopy.h>

const int autoopenTime = 750;

QPixmap * folderLocked = 0;
QPixmap * folderClosed = 0;
QPixmap * folderOpen = 0;
QPixmap * fileNormal = 0;

/*****************************************************************************
 *
 * Class ListItem
 *
 *****************************************************************************/

ListItem::ListItem(QListView *parent, const QString &s1, const QString& size, QPixmap * px, item_type t)
	: QListViewItem( parent, s1, size,QString::number(t)),pix(px)
{

}

ListItem::ListItem(QListViewItem *parent, const QString &s1, const QString& size, QPixmap * px, item_type t)
	: QListViewItem( parent, s1, size, QString::number(t)),pix(px)
{

}

void ListItem::setPixmap( QPixmap * p )
{
    pix = p;
    setup();
    widthChanged( 0 );
    invalidateHeight();
    repaint();
}


const QPixmap * ListItem::pixmap(int i) const
{
    if ( i )
        return 0;
    return pix;
}

/*****************************************************************************
 *
 * Class FileItem
 *
 *****************************************************************************/


FileItem::FileItem(QListView *parent, const QString &s1, const QString& apath, const QString& size, QPixmap * px)
	: ListItem(parent,s1,size,px,item_file)
{
	QFileInfo f(apath);
	AbsPath = f.absFilePath();

}
FileItem::FileItem(QListViewItem *parent, const QString &s1, const QString& apath, const QString& size, QPixmap * px)
	: ListItem(parent,s1,size,px,item_file)
{
	QFileInfo f(apath);
	AbsPath = f.absFilePath();
}


/*****************************************************************************
 *
 * Class Directory
 *
 *****************************************************************************/

Directory::Directory( QListViewItem * parent, const QString& filename, const QString& apath)
		: FileItem( parent, filename, apath, 0)
{
	if ( !QDir( apath ).isReadable())
	{
	    	setPixmap(folderLocked);
		setExpandable(FALSE);
	}
	else
		setPixmap(folderClosed);
	setType(item_dir);
}


Directory::Directory( QListView * parent, const QString& filename, const QString& apath, bool special)
    : FileItem( parent, filename, apath, 0)
{
   	setPixmap(folderClosed);
	if(special)
		setType(item_special);
	else
		setType(item_dir);
}


void Directory::setOpen( bool o )
{
		if ( o )
			setPixmap(folderOpen);
		else
			setPixmap(folderClosed);

		if ( o && !childCount() )
		{
			QDir thisDir( absFilePath() );
			if ( !thisDir.isReadable() )
			{
				setExpandable( FALSE );
				setPixmap(folderLocked);
				return;
			}

			listView()->setUpdatesEnabled( FALSE );
			const QFileInfoList * files = thisDir.entryInfoList();
			if ( files )
			{
				QFileInfoListIterator it( *files );
				QFileInfo * fi;
				QFileInfo li;
				while( (fi=it.current()) != 0 )
				{
					++it;
					if ( fi->fileName() == "." || fi->fileName() == ".." )
						; // nothing
					else if ( (fi->isDir() && fi->isReadable()) || (fi->isSymLink() && (li=QFileInfo(fi->readLink())).isDir()) && li.isReadable())
						(void) new Directory(this, fi->fileName(),fi->absFilePath());
				}
			}
			listView()->setUpdatesEnabled( TRUE );
		}
		QListViewItem::setOpen( o );
}


void Directory::setup()
{
    setExpandable( TRUE );
    QListViewItem::setup();
}

/*****************************************************************************
 *
 * Class DirectoryView
 *
 *****************************************************************************/

DirectoryView::DirectoryView( QWidget *parent, const char *name)
    : QListView( parent, name ), oldCurrent( 0 ),
       mousePressed( FALSE )
{

	setTreeStepSize( 20 );

	setAllColumnsShowFocus( TRUE );

	setSelectionMode(QListView::Extended);
}



void DirectoryView::contentsMousePressEvent( QMouseEvent* e )
{
    QListView::contentsMousePressEvent(e);
    QPoint p( contentsToViewport( e->pos() ) );
    QListViewItem *i = itemAt( p );
    if ( i )
		{
			currentdir = i;
			emit selectionChanged(i);
			mousePressed = TRUE;
		}
}


void DirectoryView::addSelectedItem(QStringList& list, QListViewItem * item)
{
	QListViewItem * child;
	if(!item)
		child = firstChild();
	else
		child = item->firstChild();
	while(child)
	{
		if(child->isSelected())
			if(QFile::exists(((FileItem *)child)->absFilePath()))
			{
				list << ((FileItem *)child)->absFilePath();
			}
		if(child->isOpen())
			addSelectedItem(list,child);
		child = child->nextSibling();
	}
}


void DirectoryView::contentsMouseMoveEvent( QMouseEvent* e )
{
	QStringList sourceList;

	if ( mousePressed && (e->pos().manhattanLength() > QApplication::startDragDistance()) )
	{
		mousePressed = FALSE;
		QListViewItem *item = itemAt(contentsToViewport(e->pos()));
		if (item && item->parent())
		{
			QFileInfo f(((FileItem *)item)->absFilePath());
			addSelectedItem(sourceList,NULL);
			QUriDrag* ud = new QUriDrag(viewport());
			ud->setFileNames( sourceList );
			if(f.isDir())
				ud->setPixmap(QPixmap::fromMimeSource("folder_new.png"),QPoint(8,8));
			else
				ud->setPixmap(QPixmap::fromMimeSource("file_new.png"),QPoint(8,8));
			ud->dragCopy();
		}
	}
}

void DirectoryView::contentsMouseReleaseEvent( QMouseEvent * )
{
    mousePressed = FALSE;
}


void DirectoryView::openDir(const QString& dir)
{
	currentdir->setOpen(TRUE);
	setSelected(currentdir,FALSE);
	QListViewItem * item = currentdir->firstChild();
	while(item && item->text(0)!=dir)
		item = item->nextSibling();
	if(item)
	{
		setSelected(item,TRUE);
		currentdir = item;
		setCurrentItem(item);
	}
}

void DirectoryView::closeDir()
{
	setSelected(currentdir,FALSE);
	currentdir = currentdir->parent();
	setSelected(currentdir,TRUE);
	setCurrentItem(currentdir);
}

/*****************************************************************************
 *
 * Class DirView
 *
 *****************************************************************************/

void dirView::displayFolder(QListViewItem * newselect)
{
	qdisplayFolder(((FileItem *)newselect)->absFilePath());
}

void dirView::qdisplayFolder(QString path)
{
	QDir thisDir(path, QString::null,QDir::Name | QDir::IgnoreCase | QDir::DirsFirst, QDir::All);
	const QFileInfoList * files = thisDir.entryInfoList();
	if ( files )
	{
		QFileInfoListIterator it( *files );
		QFileInfo * fi;
		QFileInfo li;
		clear();
		while( (fi=it.current()) != 0 )
		{
			++it;
			if(fi->fileName() == ".")
				;
			else if ( fi->fileName() == ".." && QDir::cleanDirPath(path)	!= "/")
				(void) new Directory(this,fi->fileName(),fi->absFilePath(),TRUE);
			else if( fi->fileName() == "..")
				;
			else if ( (fi->isDir() && fi->isReadable()) ||  (fi->isSymLink() && (li=QFileInfo(fi->readLink())).isDir()) && li.isReadable())
					(void)new Directory( this,fi->fileName(),fi->absFilePath());
			else
				(void) new FileItem( this, fi->fileName(),fi->absFilePath(),QString::number(fi->size()),fileNormal);
		}
	}
}

void dirView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
{
	QListView::contentsMouseDoubleClickEvent (e);

	QListViewItem *item = itemAt(contentsToViewport(e->pos()));
	if(item)
	{
		if(((FileItem *)item)->type() == item_dir)
			emit newdir(((FileItem *)item)->name());
		if(((FileItem *)item)->type() == item_special)
			emit closedir();
		qdisplayFolder(((FileItem *)item)->absFilePath());
	}
}

void dirView::contentsMouseMoveEvent( QMouseEvent* e )
{
	QStringList sourceList;

	if ( mousePressed && (e->pos().manhattanLength() > QApplication::startDragDistance()) )
	{
		mousePressed = FALSE;
		QListViewItem *item = itemAt(contentsToViewport(e->pos()));
		if (item && item->isSelected())
		{
			QFileInfo f(((FileItem *)item)->absFilePath());
			addSelectedItem(sourceList,NULL);
			QUriDrag* ud = new QUriDrag(viewport());
			ud->setFileNames(sourceList);
			if(f.isDir())
				ud->setPixmap(QPixmap::fromMimeSource("folder_new.png"),QPoint(8,8));
			else
				ud->setPixmap(QPixmap::fromMimeSource("file_new.png"),QPoint(8,8));
			ud->dragCopy();
		}
	}
}
